Package: dearseq
Type: Package
Title: Differential Expression Analysis for RNA-seq data through a
        robust variance component test
Version: 1.25.0
Date: 2023-06-16
Authors@R: c(person("Denis", "Agniel", role = c("aut"),
                    email = "denis.agniel@gmail.com"),
	           person("Boris P.", "Hejblum", role = c("aut", "cre"),
	                  email = "boris.hejblum@u-bordeaux.fr",
	                  comment = c(ORCID="0000-0003-0646-452X")),
	           person("Marine", "Gauthier", role = c("aut"),
	                  email = "marine.gauthier@u-bordeaux.fr"),
	           person("Mélanie", "Huchon", role = c("ctb"),
	                  email = "melanie.huchon@u-bordeaux.fr"))
Depends: R (>= 3.6.0)
Imports: CompQuadForm, dplyr, ggplot2, KernSmooth, magrittr,
        matrixStats, methods, patchwork, parallel, pbapply, reshape2,
        rlang, scattermore, stats, statmod, survey, tibble, viridisLite
Suggests: Biobase, BiocManager, BiocSet, edgeR, DESeq2, GEOquery, GSA,
        knitr, limma, readxl, rmarkdown, S4Vectors,
        SummarizedExperiment, testthat, covr
Description: Differential Expression Analysis RNA-seq data with variance 
             component score test accounting for data heteroscedasticity through
             precision weights. Perform both gene-wise and gene set analyses, 
             and can deal with repeated or longitudinal data. Methods are 
             detailed in: i) Agniel D & Hejblum BP (2017) Variance component score 
             test for time-course gene set analysis of longitudinal RNA-seq 
             data, Biostatistics, 18(4):589-604 ; and ii) Gauthier M, Agniel D, 
             Thiébaut R & Hejblum BP (2020) dearseq: a variance component 
             score test for RNA-Seq differential analysis that effectively 
             controls the false discovery rate, NAR Genomics and Bioinformatics,
             2(4):lqaa093.
License: GPL-2 | file LICENSE
biocViews: BiomedicalInformatics, CellBiology, DifferentialExpression,
        DNASeq, GeneExpression, Genetics, GeneSetEnrichment,
        ImmunoOncology, KEGG, Regression, RNASeq, Sequencing,
        SystemsBiology, TimeCourse, Transcription, Transcriptomics
BugReports: https://github.com/borishejblum/dearseq/issues
Encoding: UTF-8
RoxygenNote: 7.2.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/dearseq
git_branch: devel
git_last_commit: 568dec0
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:15:01 UTC; biocbuild
Author: Denis Agniel [aut],
  Boris P. Hejblum [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0646-452X>),
  Marine Gauthier [aut],
  Mélanie Huchon [ctb]
Maintainer: Boris P. Hejblum <boris.hejblum@u-bordeaux.fr>
